import json
from wazuh_evidence_collector.wazuh_client import WazuhClient
from elasticsearch import Elasticsearch
from elasticsearch_dsl import Search
from forward_evidence.forward_evidence import ForwardEvidence
from forward_evidence.generate_evidence import create_resource, create_assessevidence_request, print_evidence
from wazuh_evidence_collector.checker import Checker
from wazuh_evidence_collector.demo_checker import DemoChecker
import uuid
import configparser
import logging.config

f = open('constants.json',)
CONSTANTS = json.load(f)
f.close()

logging.config.fileConfig('logging.conf')
LOGGER = logging.getLogger('root')

DEMO = CONSTANTS["general"]["demo"]

if not DEMO:
    wc = WazuhClient(CONSTANTS['wazuh']['host'], CONSTANTS['wazuh']['port'], CONSTANTS['wazuh']['username'], CONSTANTS['wazuh']['password'])

    es = Elasticsearch(
            CONSTANTS['elastic']['host'],
            http_auth=(CONSTANTS['elastic']['username'], CONSTANTS['elastic']['password']),
            scheme='https',
            port=CONSTANTS['elastic']['port'],
            use_ssl=False,
            verify_certs=False,
            ssl_show_warn=False,
        )

forwarder = ForwardEvidence(CONSTANTS, LOGGER)

# Get ID (UUID)
def get_id():
    id = uuid.uuid1()

    return id

# Get tool ID (SERVICE:VERSION format)
def get_tool_id():
    with open('MANIFEST', 'r') as f:
        config_string = '[clouditor]\n' + f.read()

    config = configparser.ConfigParser()
    config.read_string(config_string)

    version = '{}:{}'.format(config.get('clouditor', 'SERVICE'), config.get('clouditor', 'VERSION'))

    return version

def main():
    try:
        run_collector()
    except BaseException as e:
        LOGGER.exception("Exception caught in run_collector()")

# Wrapper function that runs all the checks (for every manager/agent)
def run_collector():
    checker = Checker(wc, es) if not DEMO else DemoChecker()

    # Get list of all agent ids (including manager's)
    def get_agents(wc):
        body = wc.req('GET', 'agents')

        agent_list = []
        for agent in body['data']['affected_items']:
            agent_list.append([agent['id'], agent['name']])

        return body, agent_list

    body, agent_list = get_agents(wc) if not DEMO else ({}, [[0, "dummyAgent0"], [1, "dummyAgent1"]])

    ae_req_list = []

    for agent in agent_list:
        ae_req_list.append(generate_evidence(agent, checker))

    # TODO:
    for ae_req in ae_req_list:
        forwarder.send_evidence(ae_req)
        print_evidence(LOGGER, ae_req.evidence)

    return ae_req_list

# Run checks and generate evidence
def generate_evidence(agent, checker):
    raw_evidence = []

    evidence, result_syscheck = checker.check_syscheck(agent)
    raw_evidence.append(evidence)

    evidence, result_rootcheck = checker.check_rootcheck(agent)
    raw_evidence.append(evidence)

    evidence, result_alert_integration = checker.check_alert_integrations()
    raw_evidence.append(evidence)

    evidence, result_virus_total = checker.check_virus_total_integration()
    raw_evidence.append(evidence)

    evidence, result_clamd_process = checker.check_clamd_process(agent)
    raw_evidence.append(evidence)

    evidence, result_clamd_logs = checker.check_clamd_logs_elastic(agent)
    raw_evidence.append(evidence)

    # TODO: 
    if result_syscheck and result_rootcheck and result_alert_integration and \
        (result_virus_total or (result_clamd_process and result_clamd_logs)):
        malware_protection = { "malwareProtection": { "enabled": True }}
    else: 
        malware_protection = { "malwareProtection": { "enabled": False }}

    # TODO: change ID
    resource = create_resource(agent[0], agent[1], None, malware_protection)
    return create_assessevidence_request(get_id(), "evidence_collector_service", get_tool_id(), raw_evidence, resource)

if __name__ == "__main__":
    main()